# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2019-02-23 05:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=64, verbose_name='title')),
                ('thumb_media_id', models.CharField(max_length=64, verbose_name='thumb_media_id')),
                ('show_cover_pic', models.BooleanField(default=True, verbose_name='show cover')),
                ('author', models.CharField(blank=True, default='', max_length=24, null=True, verbose_name='author')),
                ('digest', models.CharField(blank=True, default='', max_length=256, null=True, verbose_name='digest')),
                ('content', models.TextField(verbose_name='content')),
                ('url', models.CharField(max_length=256, verbose_name='url')),
                ('content_source_url', models.CharField(max_length=256, verbose_name='content source url')),
                ('need_open_comment', models.NullBooleanField(default=None, verbose_name='need open comment')),
                ('only_fans_can_comment', models.NullBooleanField(default=None, verbose_name='only fans can comment')),
                ('index', models.PositiveSmallIntegerField(verbose_name='index')),
                ('_thumb_url', models.CharField(db_column='thumb_url', default=None, max_length=256, null=True)),
                ('synced_at', models.DateTimeField(auto_now_add=True, verbose_name='synchronized at')),
            ],
            options={
                'verbose_name': 'article',
                'verbose_name_plural': 'articles',
                'ordering': ('material', 'index'),
            },
        ),
        migrations.CreateModel(
            name='Material',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('image', 'IMAGE'), ('news', 'NEWS'), ('video', 'VIDEO'), ('voice', 'VOICE')], max_length=5, verbose_name='type')),
                ('media_id', models.CharField(max_length=64, verbose_name='media_id')),
                ('name', models.CharField(blank=True, max_length=64, null=True, verbose_name='name')),
                ('url', models.CharField(editable=False, max_length=512, null=True, verbose_name='url')),
                ('update_time', models.IntegerField(editable=False, null=True, verbose_name='update time')),
                ('comment', models.TextField(blank=True, verbose_name='comment')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
            ],
            options={
                'verbose_name': 'material',
                'verbose_name_plural': 'materials',
                'ordering': ('app', '-update_time'),
            },
        ),
        migrations.CreateModel(
            name='Menu',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=16, verbose_name='name')),
                ('menuid', models.IntegerField(blank=True, null=True, verbose_name='menuid')),
                ('type', models.CharField(blank=True, choices=[('click', 'CLICK'), ('miniprogram', 'MINIPROGRAM'), ('view', 'VIEW')], max_length=20, null=True, verbose_name='type')),
                ('content', jsonfield.fields.JSONField()),
                ('weight', models.IntegerField(default=0, verbose_name='weight')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
            ],
            options={
                'verbose_name': 'menu',
                'verbose_name_plural': 'menus',
                'ordering': ('app', '-weight', 'id'),
            },
        ),
        migrations.CreateModel(
            name='MessageHandler',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60, verbose_name='name')),
                ('src', models.PositiveSmallIntegerField(choices=[(2, 'wechat'), (0, 'self'), (1, 'menu')], default=0, editable=False)),
                ('strategy', models.CharField(choices=[('none', 'NONE'), ('random_one', 'RANDOM'), ('reply_all', 'REPLYALL')], default='reply_all', max_length=10, verbose_name='strategy')),
                ('flags', models.IntegerField(default=False, verbose_name='flags')),
                ('starts', models.DateTimeField(blank=True, null=True, verbose_name='starts')),
                ('ends', models.DateTimeField(blank=True, null=True, verbose_name='ends')),
                ('enabled', models.BooleanField(default=True, verbose_name='enabled')),
                ('weight', models.IntegerField(default=0, verbose_name='weight')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated_at')),
            ],
            options={
                'verbose_name': 'message handler',
                'verbose_name_plural': 'message handlers',
                'ordering': ('-weight', '-created_at', '-id'),
            },
        ),
        migrations.CreateModel(
            name='MessageLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('msg_id', models.BigIntegerField(null=True, verbose_name='msgid')),
                ('type', models.CharField(choices=[('event', 'EVENT'), ('image', 'IMAGE'), ('link', 'LINK'), ('location', 'LOCATION'), ('shortvideo', 'SHORTVIDEO'), ('text', 'TEXT'), ('video', 'VIDEO'), ('voice', 'VOICE')], max_length=24, verbose_name='message type')),
                ('content', jsonfield.fields.JSONField(verbose_name='content')),
                ('direct', models.BooleanField(default=False, verbose_name='direct')),
                ('raw', models.TextField(blank=True, default=None, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
            ],
            options={
                'verbose_name': 'message log',
                'verbose_name_plural': 'message logs',
                'ordering': ('app', '-created_at'),
            },
        ),
        migrations.CreateModel(
            name='Reply',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('custom', 'CUSTOM'), ('forward', 'FORWARD'), ('image', 'IMAGE'), ('music', 'MUSIC'), ('news', 'NEWS'), ('text', 'TEXT'), ('video', 'VIDEO'), ('voice', 'VOICE')], db_column='type', max_length=16, verbose_name='type')),
                ('_content', jsonfield.fields.JSONField(db_column='content', default=dict)),
                ('weight', models.IntegerField(default=0, verbose_name='weight')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated_at')),
                ('handler', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='replies', to='wechat_django.MessageHandler')),
            ],
            options={
                'verbose_name': 'reply',
                'verbose_name_plural': 'replies',
                'ordering': ('-weight', 'id'),
            },
        ),
        migrations.CreateModel(
            name='Rule',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('all', 'ALL'), ('contain', 'CONTAIN'), ('equal', 'EQUAL'), ('event', 'EVENT'), ('eventkey', 'EVENTKEY'), ('msg_type', 'MSGTYPE'), ('regex', 'REGEX')], max_length=16, verbose_name='type')),
                ('_content', jsonfield.fields.JSONField(blank=True, db_column='content')),
                ('weight', models.IntegerField(default=0, verbose_name='weight')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated_at')),
                ('handler', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='rules', to='wechat_django.MessageHandler')),
            ],
            options={
                'verbose_name': 'rule',
                'verbose_name_plural': 'rules',
                'ordering': ('-weight', 'id'),
            },
        ),
        migrations.CreateModel(
            name='WeChatApp',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(help_text='公众号名称,用于后台辨识公众号', max_length=16, verbose_name='title')),
                ('name', models.CharField(help_text='公众号唯一标识,可采用微信号,设定后不可修改,用于程序辨识', max_length=16, unique=True, verbose_name='name')),
                ('desc', models.TextField(blank=True, default='', verbose_name='description')),
                ('appid', models.CharField(max_length=32, verbose_name='AppId')),
                ('appsecret', models.CharField(max_length=64, null=True, verbose_name='AppSecret')),
                ('type', models.PositiveSmallIntegerField(choices=[(3, 'MINIPROGRAM'), (1, 'SERVICEAPP'), (2, 'SUBSCRIBEAPP')], default=1, verbose_name='type')),
                ('token', models.CharField(blank=True, max_length=32, null=True)),
                ('encoding_aes_key', models.CharField(blank=True, max_length=43, null=True, verbose_name='EncodingAESKey')),
                ('encoding_mode', models.PositiveSmallIntegerField(choices=[(0, 'PLAIN'), (2, 'SAFE')], default=0, verbose_name='encoding mode')),
                ('flags', models.IntegerField(default=0, verbose_name='flags')),
                ('ext_info', jsonfield.fields.JSONField(db_column='ext_info', default={})),
                ('configurations', jsonfield.fields.JSONField(db_column='configurations', default={})),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
            ],
            options={
                'verbose_name': 'WeChat app',
                'verbose_name_plural': 'WeChat apps',
            },
        ),
        migrations.CreateModel(
            name='WeChatUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('openid', models.CharField(max_length=36, verbose_name='openid')),
                ('unionid', models.CharField(max_length=36, null=True, verbose_name='unionid')),
                ('nickname', models.CharField(max_length=24, null=True, verbose_name='nickname')),
                ('sex', models.SmallIntegerField(choices=[(2, 'FEMALE'), (1, 'MALE'), (0, 'UNKNOWN')], null=True, verbose_name='gender')),
                ('headimgurl', models.CharField(max_length=256, null=True, verbose_name='avatar')),
                ('city', models.CharField(max_length=24, null=True, verbose_name='city')),
                ('province', models.CharField(max_length=24, null=True, verbose_name='province')),
                ('country', models.CharField(max_length=24, null=True, verbose_name='country')),
                ('language', models.CharField(max_length=24, null=True, verbose_name='language')),
                ('subscribe', models.NullBooleanField(verbose_name='is subscribed')),
                ('subscribe_time', models.IntegerField(null=True, verbose_name='subscribe time')),
                ('subscribe_scene', models.CharField(choices=[('ADD_SCENE_ACCOUNT_MIGRATION', 'ADD_SCENE_ACCOUNT_MIGRATION'), ('ADD_SCENE_OTHERS', 'ADD_SCENE_OTHERS'), ('ADD_SCENE_PAID', 'ADD_SCENE_PAID'), ('ADD_SCENE_PROFILE_CARD', 'ADD_SCENE_PROFILE_CARD'), ('ADD_SCENE_PROFILE_ITEM', 'ADD_SCENE_PROFILE_ITEM'), ('ADD_SCENEPROFILE LINK', 'ADD_SCENE_PROFILE_LINK'), ('ADD_SCENE_QR_CODE', 'ADD_SCENE_QR_CODE'), ('ADD_SCENE_SEARCH', 'ADD_SCENE_SEARCH')], max_length=32, null=True, verbose_name='subscribe scene')),
                ('qr_scene', models.IntegerField(null=True, verbose_name='qr scene')),
                ('qr_scene_str', models.CharField(max_length=64, null=True, verbose_name='qr_scene_str')),
                ('remark', models.CharField(blank=True, max_length=30, null=True, verbose_name='WeChat remark')),
                ('comment', models.TextField(blank=True, null=True, verbose_name='remark')),
                ('groupid', models.IntegerField(null=True, verbose_name='group id')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='updated at')),
                ('synced_at', models.DateTimeField(default=None, null=True, verbose_name='synchronized at')),
                ('app', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='users', to='wechat_django.WeChatApp')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'ordering': ('app', '-created_at'),
            },
        ),
        migrations.AddField(
            model_name='messagelog',
            name='app',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wechat_django.WeChatApp'),
        ),
        migrations.AddField(
            model_name='messagelog',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='wechat_django.WeChatUser'),
        ),
        migrations.AddField(
            model_name='messagehandler',
            name='app',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='message_handlers', to='wechat_django.WeChatApp'),
        ),
        migrations.AddField(
            model_name='menu',
            name='app',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='menus', to='wechat_django.WeChatApp'),
        ),
        migrations.AddField(
            model_name='menu',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='sub_button', to='wechat_django.Menu'),
        ),
        migrations.AddField(
            model_name='material',
            name='app',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='materials', to='wechat_django.WeChatApp'),
        ),
        migrations.AddField(
            model_name='article',
            name='material',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='articles', to='wechat_django.Material'),
        ),
        migrations.AlterUniqueTogether(
            name='wechatuser',
            unique_together=set([('app', 'unionid'), ('app', 'openid')]),
        ),
        migrations.AlterIndexTogether(
            name='messagelog',
            index_together=set([('app', 'created_at')]),
        ),
        migrations.AlterIndexTogether(
            name='messagehandler',
            index_together=set([('app', 'weight', 'created_at')]),
        ),
        migrations.AlterUniqueTogether(
            name='material',
            unique_together=set([('app', 'media_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='article',
            unique_together=set([('material', 'index')]),
        ),
    ]



# -*- coding: utf-8 -*-
# flake8: noqa
from __future__ import unicode_literals

from .wechat import default_site as wechat, WeChatSite
from .admin import default_site as admin, WeChatAdminSite, WeChatAdminSiteMixin



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _

from . import settings


class WeChatConfig(AppConfig):
    name = "wechat_django"
    verbose_name = _("WeChat")
    verbose_name_plural = _("WeChat")

    def ready(self):
        import object_tool
        object_tool.ObjectToolConfig.register()

        if settings.PATCHADMINSITE:
            from .sites.admin import patch_admin
            patch_admin()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from wechatpy import events, messages

from ..models import MessageHandler, Rule
from .base import WeChatTestCase


class HandlerTestCase(WeChatTestCase):
    def test_match(self):
        """测试匹配"""
        def _create_msg(type, **kwargs):
            rv = type(dict())
            for k, v in kwargs.items():
                setattr(rv, k, v)
            return rv

        content = "某中文"
        text_message = _create_msg(messages.TextMessage, content=content)
        another_content = "abc"
        another_text_message = _create_msg(messages.TextMessage,
            content=another_content)

        media_id = "media_id"
        url = "http://example.com/foo?bar=1"
        image_message = _create_msg(messages.ImageMessage, media_id=media_id, image=url)

        event_key = "key"
        sub_event = _create_msg(events.SubscribeEvent, key=event_key)
        click_event = _create_msg(events.ClickEvent, key=event_key)
        another_key = "another"
        another_click_event = _create_msg(events.ClickEvent, key=another_key)

        # 所有消息
        rule = Rule(type=Rule.Type.ALL)
        self.assertMatch(rule, text_message)
        self.assertMatch(rule, image_message)

        # 测试类型匹配
        rule = Rule(type=
            Rule.Type.MSGTYPE, msg_type=Rule.ReceiveMsgType.IMAGE)
        self.assertNotMatch(rule, text_message)
        self.assertMatch(rule, image_message)

        # 测试事件匹配
        rule = Rule(type=
            Rule.Type.EVENT, event=MessageHandler.EventType.SUBSCRIBE)
        self.assertNotMatch(rule, text_message)
        self.assertMatch(rule, sub_event)
        self.assertNotMatch(rule, click_event)

        # 测试指定事件匹配
        rule = Rule(type=
            Rule.Type.EVENTKEY, event=MessageHandler.EventType.CLICK,
            key=event_key)
        self.assertNotMatch(rule, text_message)
        self.assertNotMatch(rule, sub_event)
        self.assertMatch(rule, click_event)
        self.assertNotMatch(rule, another_click_event)

        # 测试包含匹配
        rule = Rule(type=Rule.Type.CONTAIN, pattern="中")
        self.assertMatch(rule, text_message)
        self.assertNotMatch(rule, another_text_message)
        self.assertNotMatch(rule, image_message)
        self.assertNotMatch(rule, click_event)

        # 测试相等匹配
        rule = Rule(type=Rule.Type.EQUAL, pattern=content)
        self.assertMatch(rule, text_message)
        self.assertNotMatch(rule, another_text_message)
        self.assertNotMatch(rule, image_message)
        self.assertNotMatch(rule, click_event)

        # 测试正则匹配
        rule = Rule(type=Rule.Type.REGEX, pattern=r"[a-c]+")
        self.assertNotMatch(rule, text_message)
        self.assertMatch(rule, another_text_message)
        self.assertNotMatch(rule, image_message)
        self.assertNotMatch(rule, click_event)

        # 测试handler匹配
        handler3 = self._create_handler(rules=[dict(
            type=Rule.Type.EQUAL,
            pattern=content
        ), dict(
            type=Rule.Type.EQUAL,
            pattern=another_content
        )], name="3")
        self.assertTrue(handler3.is_match(
            self._msg2info(text_message)))
        self.assertTrue(handler3.is_match(
            self._msg2info(another_text_message)))
        self.assertFalse(handler3.is_match(
            self._msg2info(click_event)))

        # 测试匹配顺序
        handler1 = self._create_handler(rules=[dict(
            type=Rule.Type.EVENTKEY,
            event=MessageHandler.EventType.CLICK,
            key=event_key
        )], name="1", weight=5)
        handler2 = self._create_handler(rules=[dict(
            type=Rule.Type.EQUAL,
            pattern=content
        )], name="2")
        handler4 = self._create_handler(rules=[dict(
            type=Rule.Type.EVENT,
            event=MessageHandler.EventType.CLICK
        )], name="4", weight=-5)
        matches = MessageHandler.matches(self._msg2info(text_message))
        self.assertEqual(len(matches), 1)
        self.assertEqual(matches[0].id, handler2.id)
        matches = MessageHandler.matches(self._msg2info(click_event))
        self.assertEqual(len(matches), 1)
        self.assertEqual(matches[0].id, handler1.id)
        matches = MessageHandler.matches(self._msg2info(another_click_event))
        self.assertEqual(len(matches), 1)
        self.assertEqual(matches[0].id, handler4.id)

    def assertMatch(self, rule, message):
        self.assertTrue(rule._match(message))

    def assertNotMatch(self, rule, message):
        self.assertFalse(rule._match(message))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.contrib import messages
from django.urls import reverse
from django.utils.http import urlencode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
import object_tool
from wechatpy.exceptions import WeChatClientException

from ...models import Menu
from ..utils import get_request_params
from ..base import DynamicChoiceForm, WeChatModelAdmin


class MenuAdmin(WeChatModelAdmin):
    __category__ = "menu"
    __model__ = Menu

    changelist_object_tools = ("sync", "publish")
    change_form_template = "admin/wechat_django/menu/change_form.html"
    change_list_template = "admin/wechat_django/menu/change_list.html"

    list_display = (
        "operates", "id", "parent_id", "title", "type", "detail", "weight",
        "updated_at")
    list_display_links = ("title",)
    list_editable = ("weight", )
    fields = (
        "name", "type", "key", "url", "appid", "pagepath", "created_at",
        "updated_at")

    def title(self, obj):
        if obj.parent:
            return "|--- " + obj.name
        return obj.name
    title.short_description = _("title")

    @mark_safe
    def detail(self, obj):
        rv = ""
        if obj.type == Menu.Event.CLICK:
            rv = obj.content.get("key")
        elif obj.type == Menu.Event.VIEW:
            rv = '<a href="{0}">{1}</a>'.format(
                obj.content.get("url"), _("link"))
        elif obj.type == Menu.Event.MINIPROGRAM:
            rv = obj.content.get("appid")
        return rv or ""
    detail.short_description = _("detail")

    @mark_safe
    def operates(self, obj):
        del_url = reverse("admin:wechat_django_menu_delete", kwargs=dict(
            object_id=obj.id,
            wechat_app_id=obj.app_id
        ))
        rv = '<a class="deletelink" href="{0}"></a>'.format(del_url)
        if not obj.parent and not obj.type and obj.sub_button.count() < 5:
            query = dict(parent_id=obj.id)
            add_link = reverse("admin:wechat_django_menu_add", kwargs=dict(
                wechat_app_id=obj.app_id
            ))
            add_url = "{0}?{1}".format(add_link, urlencode(query))
            rv += '<a class="addlink" href="{0}"></a>'.format(add_url)
        return rv
    operates.short_description = _("actions")

    @object_tool.confirm(short_description=_("Sync menus"))
    def sync(self, request, obj=None):
        self.check_wechat_permission(request, "sync")

        def action():
            request.app.sync_menus()
            return _("Menus successful synchronized")

        return self._clientaction(
            request, action, _("Sync menus failed with %(exc)s"))

    @object_tool.confirm(short_description=_("Publish menus"))
    def publish(self, request, obj=None):
        self.check_wechat_permission(request, "publish")

        def action():
            request.app.publish_menus()
            return _("Menus successful published")

        return self._clientaction(
            request, action, _("Publish menus failed with %(exc)s"))

    def get_actions(self, request):
        actions = super(MenuAdmin, self).get_actions(request)
        if "delete_selected" in actions:
            del actions["delete_selected"]
        return actions

    def get_fields(self, request, obj=None):
        fields = list(super(MenuAdmin, self).get_fields(request, obj))
        if not obj:
            fields.remove("created_at")
            fields.remove("updated_at")
        return fields

    def get_readonly_fields(self, request, obj=None):
        rv = super(MenuAdmin, self).get_readonly_fields(request, obj)
        if obj:
            rv = rv + ("created_at", "updated_at")
        return rv

    def get_queryset(self, request):
        rv = super(MenuAdmin, self).get_queryset(request)
        if not get_request_params(request, "menuid"):
            rv = rv.filter(menuid__isnull=True)
        if request.GET.get("parent_id"):
            rv = rv.filter(parent_id=request.GET["parent_id"])
        return rv

    class MenuForm(DynamicChoiceForm):
        content_field = "content"
        origin_fields = ("name", "menuid", "type", "weight")
        type_field = "type"

        key = forms.CharField(label=_("menu key"), required=False)
        url = forms.URLField(label=_("url"), required=False)
        appid = forms.CharField(label=_("miniprogram app_id"), required=False)
        pagepath = forms.CharField(label=_("pagepath"), required=False)

        class Meta(object):
            model = Menu
            fields = ("name", "menuid", "type", "weight")

        def allowed_fields(self, type, cleaned_data):
            if type == Menu.Event.VIEW:
                fields = ("url", )
            elif type == Menu.Event.CLICK:
                fields = ("key", )
            elif type == Menu.Event.MINIPROGRAM:
                fields = ("url", "appid", "apppath")
            else:
                fields = tuple()
            return fields
    form = MenuForm

    def save_model(self, request, obj, form, change):
        if not change and request.GET.get("parent_id"):
            obj.parent_id = request.GET["parent_id"]
        return super().save_model(request, obj, form, change)

    def has_add_permission(self, request):
        if not super(MenuAdmin, self).has_add_permission(request):
            return False
        # 判断菜单是否已满
        q = self.get_queryset(request)
        if request.GET.get("parent_id"):
            return q.count() < 5
        else:
            return q.filter(parent_id__isnull=True).count() < 3

    def get_model_perms(self, request):
        if not request.app.abilities.menus:
            return {}
        return super(MenuAdmin, self).get_model_perms(request)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
import time

from django.test import RequestFactory
from django.utils.http import urlencode
from httmock import response
from requests.exceptions import HTTPError
from six.moves.urllib.parse import parse_qsl
from wechatpy import messages, parse_message, replies
from wechatpy.utils import check_signature, WeChatSigner

from ..exceptions import MessageHandleError
from ..models import MessageHandler, Reply, WeChatMessageInfo
from ..sites.wechat import patch_request
from .base import WeChatTestCase
from .interceptors import (common_interceptor, wechatapi,
    wechatapi_accesstoken, wechatapi_error)


class ReplyTestCase(WeChatTestCase):
    def test_reply(self):
        """测试一般回复"""
        sender = "openid"
        message = messages.TextMessage(dict(
            FromUserName=sender,
            content="xyz"
        ))

        def _test_reply(type, **kwargs):
            reply = Reply(
                type=type,
                **{
                    k: v
                    for k, v in kwargs.items()
                    if v is not None
                }
            )
            reply_message = reply.normal_reply(message)
            self.assertEqual(reply_message.target, sender)
            self.assertEqual(reply_message.type, type)
            return reply_message

        # 测试文本回复
        content = "test"
        reply = _test_reply(Reply.MsgType.TEXT, content=content)
        self.assertEqual(reply.content, content)

        # 测试图片回复
        media_id = "media_id"
        reply = _test_reply(Reply.MsgType.IMAGE, media_id=media_id)
        self.assertEqual(reply.image, media_id)

        # 测试音频回复
        reply = _test_reply(Reply.MsgType.VOICE, media_id=media_id)
        self.assertEqual(reply.voice, media_id)

        # 测试视频回复
        title = "title"
        description = "desc"
        reply = _test_reply(
            Reply.MsgType.VIDEO, media_id=media_id, title=title,
            description=description)
        self.assertEqual(reply.media_id, media_id)
        self.assertEqual(reply.title, title)
        self.assertEqual(reply.description, description)
        # 选填字段
        reply = _test_reply(Reply.MsgType.VIDEO, media_id=media_id)
        self.assertEqual(reply.media_id, media_id)
        self.assertIsNone(reply.title)
        self.assertIsNone(reply.description)

        # 测试音乐回复
        music_url = "music_url"
        hq_music_url = "hq_music_url"
        reply = _test_reply(
            Reply.MsgType.MUSIC, thumb_media_id=media_id, title=title,
            description=description, music_url=music_url,
            hq_music_url=hq_music_url)
        self.assertEqual(reply.thumb_media_id, media_id)
        self.assertEqual(reply.title, title)
        self.assertEqual(reply.description, description)
        self.assertEqual(reply.music_url, music_url)
        self.assertEqual(reply.hq_music_url, hq_music_url)
        # 选填字段
        reply = _test_reply(Reply.MsgType.MUSIC, thumb_media_id=media_id)
        self.assertEqual(reply.thumb_media_id, media_id)
        self.assertIsNone(reply.title)
        self.assertIsNone(reply.description)
        self.assertIsNone(reply.music_url)
        self.assertIsNone(reply.hq_music_url)

        # 测试图文回复
        pass

    def test_multireply(self):
        """测试多回复"""
        reply1 = "abc"
        reply2 = "def"
        replies = [dict(
            type=Reply.MsgType.TEXT,
            content=reply1
        ), dict(
            type=Reply.MsgType.TEXT,
            content=reply2
        )]
        handler_all = self._create_handler(
            replies=replies, strategy=MessageHandler.ReplyStrategy.REPLYALL)
        handler_rand = self._create_handler(
            replies=replies, strategy=MessageHandler.ReplyStrategy.RANDOM)

        # 随机回复
        api = "/cgi-bin/message/custom/send"
        sender = "openid"
        message = messages.TextMessage(dict(
            FromUserName=sender,
            content="xyz"
        ))
        message = self._wrap_message(message)
        with wechatapi_accesstoken(), wechatapi_error(api):
            reply = handler_rand.reply(message)
            self.assertEqual(reply.type, Reply.MsgType.TEXT)
            self.assertEqual(reply.target, sender)
            self.assertIn(reply.content, (reply1, reply2))

        # 回复一条正常消息以及一条客服消息
        counter = dict(calls=0)

        def callback(url, request, response):
            counter["calls"] += 1
            data = json.loads(request.body.decode())
            self.assertEqual(data["text"]["content"], reply1)
            self.assertEqual(data["touser"], sender)
        with wechatapi_accesstoken(), wechatapi(api, dict(errcode=0, errmsg=""), callback):
            reply = handler_all.reply(message)
            self.assertEqual(reply.type, Reply.MsgType.TEXT)
            self.assertEqual(reply.target, sender)
            self.assertEqual(reply.content, reply2)
            self.assertEqual(counter["calls"], 1)

    def test_custom(self):
        """测试自定义回复"""
        from ..models import WeChatApp

        def _get_handler(handler, app=None):
            return self._create_handler(replies=dict(
                type=Reply.MsgType.CUSTOM,
                program="wechat_django.tests.test_model_handler." + handler
            ), app=app)

        sender = "openid"
        message = messages.TextMessage(dict(
            FromUserName=sender,
            content="xyz"
        ))
        message = self._wrap_message(message)
        success_reply = "success"
        # 测试自定义回复
        handler = _get_handler("debug_handler")
        reply = handler.reply(message)
        self.assertIsInstance(reply, replies.TextReply)
        self.assertEqual(reply.content, success_reply)

        # 测试未加装饰器的自定义回复
        handler = _get_handler("forbidden_handler")
        self.assertRaises(MessageHandleError, lambda: handler.reply(message))

        # 测试不属于本app的自定义回复
        handler_success = _get_handler("app_only_handler")
        handler_fail = _get_handler("app_only_handler",
            WeChatApp.objects.get_by_name("test1"))
        reply = handler_success.reply(message)
        self.assertIsInstance(reply, replies.TextReply)
        self.assertEqual(reply.content, success_reply)
        message._app = WeChatApp.objects.get_by_name("test1")
        self.assertRaises(MessageHandleError, lambda: handler_fail.reply(message))

    def test_forward(self):
        """测试转发回复"""
        scheme = "http"
        netloc = "example.com"
        path = "/debug"
        url = "{scheme}://{netloc}{path}".format(
            scheme=scheme,
            netloc=netloc,
            path=path
        )

        token = self.app.token
        timestamp = str(int(time.time()))
        nonce = "123456"
        query_data = dict(
            timestamp=timestamp,
            nonce=nonce
        )
        signer = WeChatSigner()
        signer.add_data(token, timestamp, nonce)
        signature = signer.signature
        query_data["signature"] = signature

        sender = "openid"
        content = "xyz"
        xml = """<xml>
        <ToUserName><![CDATA[toUser]]></ToUserName>
        <FromUserName><![CDATA[{sender}]]></FromUserName>
        <CreateTime>1348831860</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[{content}]]></Content>
        <MsgId>1234567890123456</MsgId>
        </xml>""".format(sender=sender, content=content)
        req_url = url + "?" + urlencode(query_data)
        request = RequestFactory().post(req_url, xml, content_type="text/xml")

        patch_request(request, self.app.name, WeChatMessageInfo)
        message = request.wechat

        reply_text = "abc"

        def reply_test(url, request):
            self.assertEqual(url.scheme, scheme)
            self.assertEqual(url.netloc, netloc)
            self.assertEqual(url.path, path)

            query = dict(parse_qsl(url.query))
            self.assertEqual(query["timestamp"], timestamp)
            self.assertEqual(query["nonce"], nonce)
            self.assertEqual(query["signature"], signature)
            check_signature(self.app.token, query["signature"], timestamp, nonce)

            msg = parse_message(request.body)
            self.assertIsInstance(msg, messages.TextMessage)
            self.assertEqual(msg.source, sender)
            self.assertEqual(msg.content, content)
            reply = replies.create_reply(reply_text, msg)
            return response(content=reply.render())

        handler = self._create_handler(replies=dict(
            type=Reply.MsgType.FORWARD,
            url=url
        ))

        with common_interceptor(reply_test):
            reply = handler.reply(message)
            self.assertIsInstance(reply, replies.TextReply)
            self.assertEqual(reply.content, reply_text)
            self.assertEqual(reply.target, sender)

        def bad_reply(url, request):
            return response(404)

        with common_interceptor(bad_reply):
            self.assertRaises(HTTPError, lambda: handler.reply(message))

    def test_send(self):
        """测试客服回复"""
        sender = "openid"
        message = messages.TextMessage(dict(
            FromUserName=sender,
            content="xyz"
        ))
        message = self._wrap_message(message)

        # 空消息转换
        empty_msg = replies.EmptyReply()
        empty_str = ""
        self.assertIsNone(Reply.reply2send(empty_msg)[0])
        self.assertIsNone(Reply.reply2send(empty_str)[0])

        client = self.app.client.message

        # 文本消息转换
        content = "test"
        msg_type = Reply.MsgType.TEXT
        reply = Reply(type=msg_type, content=content).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_text")
        self.assertEqual(reply.content, kwargs["content"])

        # 图片消息转换
        media_id = "media_id"
        msg_type = Reply.MsgType.IMAGE
        reply = Reply(type=msg_type, media_id=media_id).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_image")
        self.assertEqual(reply.media_id, kwargs["media_id"])

        # 声音消息转换
        msg_type = Reply.MsgType.VOICE
        reply = Reply(type=msg_type, media_id=media_id).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_voice")
        self.assertEqual(reply.media_id, kwargs["media_id"])

        # 视频消息转换
        title = "title"
        description = "desc"
        msg_type = Reply.MsgType.VIDEO
        reply = Reply(type=msg_type, media_id=media_id, title=title,
            description=description).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_video")
        self.assertEqual(reply.media_id, kwargs["media_id"])
        self.assertEqual(reply.title, kwargs["title"])
        self.assertEqual(reply.description, kwargs["description"])
        # 选填字段
        reply = Reply(type=msg_type, media_id=media_id).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_video")
        self.assertEqual(reply.media_id, kwargs["media_id"])
        self.assertIsNone(kwargs["title"])
        self.assertIsNone(kwargs["description"])

        # 音乐消息转换
        music_url = "music_url"
        hq_music_url = "hq_music_url"
        msg_type = Reply.MsgType.MUSIC
        reply = Reply(type=msg_type, thumb_media_id=media_id, title=title,
            description=description, music_url=music_url,
            hq_music_url=hq_music_url).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_music")
        self.assertEqual(reply.thumb_media_id, kwargs["thumb_media_id"])
        self.assertEqual(reply.music_url, kwargs["url"])
        self.assertEqual(reply.hq_music_url, kwargs["hq_url"])
        self.assertEqual(reply.title, kwargs["title"])
        self.assertEqual(reply.description, kwargs["description"])
        # 选填字段
        reply = Reply(type=msg_type, thumb_media_id=media_id).reply(message)
        funcname, kwargs = Reply.reply2send(reply)
        self.assertTrue(hasattr(client, funcname))
        self.assertEqual(funcname, "send_music")
        self.assertEqual(reply.thumb_media_id, kwargs["thumb_media_id"])
        self.assertIsNone(kwargs["url"])
        self.assertIsNone(kwargs["hq_url"])
        self.assertIsNone(kwargs["title"])
        self.assertIsNone(kwargs["description"])

        # 图文消息转换
        pass

        # 确认消息发送
        handler = self._create_handler(replies=dict(
            type=Reply.MsgType.TEXT,
            content=content
        ))

        def callback(url, request, response):
            data = json.loads(request.body.decode())
            self.assertEqual(data["touser"], sender)
            self.assertEqual(data["msgtype"], Reply.MsgType.TEXT)
            self.assertEqual(data["text"]["content"], content)

        with wechatapi_accesstoken(), wechatapi("/cgi-bin/message/custom/send", dict(
            errcode=0
        ), callback):
            handler.replies.all()[0].send(message)

    def _wrap_message(self, message):
        return WeChatMessageInfo(
            _app=self.app,
            _message=message
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import messages
from django.urls import reverse
from django.utils.http import urlencode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
import object_tool
from wechatpy.exceptions import WeChatClientException

from ...constants import AppType
from ...models import UserTag
from ..utils import field_property
from ..base import RecursiveDeleteActionMixin, WeChatModelAdmin


class UserTagAdmin(RecursiveDeleteActionMixin, WeChatModelAdmin):
    __category__ = "usertag"
    __model__ = UserTag

    actions = ("sync_users", "sync_openids")
    changelist_object_tools = ("sync",)
    list_display = ("id",  "name", "sys_tag", "count", "created_at")
    search_fields = ("name", )

    fields = list_display
    readonly_fields = ("id", "count", "sys_tag")

    sys_tag = field_property("sys_tag", boolean=True, short_description=_("sys tag"))

    @object_tool.confirm(short_description=_("Sync user tags"))
    def sync(self, request, obj=None):
        self.check_wechat_permission(request, "sync")
        def action():
            tags = UserTag.sync(request.app)
            msg = _("%(count)d tags successfully synchronized")
            return msg % dict(count=len(tags))

        return self._clientaction(
            request, action, _("Sync user tags failed with %(exc)s"))

    def sync_users(self, request, queryset, detail=True):
        self.check_wechat_permission(request, "sync", "user")
        def action():
            tags = queryset.all()
            for tag in tags:
                users = tag.sync_users(detail)
                msg = _("%(count)d users of %(tag)s successfully synchronized")
                return msg % dict(count=len(users), tag=tag.name)
        
        return self._clientaction(
            request, action, _("Sync users failed with %(exc)s"))
    sync_users.short_description = _("sync tag users")

    sync_openids = lambda self, request, queryset: self.sync_users(
        request, queryset, False)
    sync_openids.short_description = _("sync tag openids")

    @mark_safe
    def count(self, obj):
        return '<a href="{link}">{count}</a>'.format(
            link="{0}?{1}".format(
                reverse(
                    "admin:wechat_django_wechatuser_changelist",
                    kwargs=dict(wechat_app_id=obj.app_id)
                ),
                urlencode(dict(
                    tags__in=obj._id
                ))
            ),
            count=obj.users.count()
        )
    count.short_description = _("users count")

    def get_fields(self, request, obj=None):
        fields = list(super(UserTagAdmin, self).get_fields(request, obj))
        if not obj:
            fields.remove("count")
            fields.remove("created_at")
        return fields

    def get_readonly_fields(self, request, obj=None):
        rv = super(UserTagAdmin, self).get_readonly_fields(request, obj)
        if obj:
            rv = rv + ("created_at",)
        return rv

    def has_delete_permission(self, request, obj=None):
        rv = super(UserTagAdmin, self).has_delete_permission(request, obj)
        if rv and obj:
            return not obj.sys_tag
        return rv

    def has_add_permission(self, request):
        return super(UserTagAdmin, self).has_add_permission(request)\
            and self.get_queryset(request).exclude(
                id__in=UserTag.SYS_TAGS).count() < 100

    def get_model_perms(self, request):
        if request.app.type not in (AppType.SERVICEAPP, AppType.SUBSCRIBEAPP):
            return {}
        return super(UserTagAdmin, self).get_model_perms(request)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class AppType(object):
    OTHER = 0
    SERVICEAPP = 1
    SUBSCRIBEAPP = 2
    MINIPROGRAM = 4


class WeChatSNSScope(object):
    BASE = "snsapi_base"
    USERINFO = "snsapi_userinfo"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from wechat_django.admin.base import has_wechat_permission
from wechat_django.admin.wechatapp import WeChatAppAdmin
from wechat_django.models import WeChatApp
from ..models import WeChatPay


class WeChatPayForm(forms.ModelForm):
    clear_certs = forms.BooleanField(
        label=_("clear certs"), initial=False, required=False,
        help_text=_("Your mch_cert already uploaded"))
    _mch_cert = forms.FileField(
        label=_("mch_cert"), required=False, help_text=_("商户证书"))
    _mch_key = forms.FileField(
        label=_("mch_key"), required=False, help_text=_("商户证书私钥"))

    class Meta(object):
        model = WeChatPay
        fields = (
            "title", "name", "weight", "mch_id", "api_key", "sub_mch_id",
            "mch_app_id", "_mch_cert", "_mch_key", "clear_certs")
        widgets = dict(
            api_key=forms.PasswordInput(render_value=True),
        )

    def __init__(self, *args, **kwargs):
        super(WeChatPayForm, self).__init__(*args, **kwargs)

        # 处理字段
        inst = self.instance
        if inst.pk:
            self._readonly_field("name")
            self._readonly_field("mch_id")
            if not inst.mch_app_id:
                self._remove_field("sub_mch_id")
                self._remove_field("mch_app_id")
        if inst.pk and inst.mch_cert and inst.mch_key:
            self._remove_field("_mch_cert")
            self._remove_field("_mch_key")
        else:
            self._remove_field("clear_certs")

    def _remove_field(self, field):
        self.fields[field].widget = forms.widgets.HiddenInput()
        self.fields[field].disabled = True

    def _readonly_field(self, field):
        self.fields[field].disabled = True

    def clean__mch_cert(self):
        file = self.cleaned_data.get("_mch_cert")
        if file:
            return file.read()
        return None

    def clean__mch_key(self):
        file = self.cleaned_data.get("_mch_key")
        if file:
            return file.read()
        return None

    def clean(self):
        rv = super(WeChatPayForm, self).clean()
        mch_cert = rv.get("_mch_cert")
        mch_key = rv.get("_mch_key")
        if (mch_cert or mch_key) and not (mch_cert and mch_key):
            self.add_error(
                "_mch_cert", _("must upload both mch_cert and mch_key"))
        return rv

    def _post_clean(self):
        super(WeChatPayForm, self)._post_clean()
        # 处理证书
        if self.cleaned_data.get("clear_certs"):
            self.instance.mch_cert = None
            self.instance.mch_key = None
        if self.cleaned_data.get("_mch_cert"):
            self.instance.mch_cert = self.cleaned_data.pop("_mch_cert")
        if self.cleaned_data.get("_mch_key"):
            self.instance.mch_key = self.cleaned_data.pop("_mch_key")


class WeChatPayInline(admin.StackedInline):
    form = WeChatPayForm
    model = WeChatPay

    def get_extra(self, request, obj=None):
        return 0 if obj.pay else 1


admin.site.unregister(WeChatApp)


@admin.register(WeChatApp)
class WeChatAppWithPayAdmin(WeChatAppAdmin):
    inlines = (WeChatPayInline,)

    def get_inline_instances(self, request, obj=None):
        rv = super(WeChatAppWithPayAdmin, self).get_inline_instances(
            request, obj)
        if not obj or obj.type not in (
            WeChatApp.Type.SERVICEAPP, WeChatApp.Type.MINIPROGRAM,
            WeChatApp.Type.OTHER):
            return []
        # 支付权限
        if not has_wechat_permission(request, obj, "pay", "manage"):
            return []
        return rv

    def get_deleted_objects(self, objs, request):
        from ..models import UnifiedOrder
        deleted_objects, model_count, perms_needed, protected =\
            super(WeChatAppWithPayAdmin, self).get_deleted_objects(objs, request)
        ignored_models = (UnifiedOrder._meta.verbose_name,)
        perms_needed = perms_needed.difference(ignored_models)
        return deleted_objects, model_count, perms_needed, protected



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from ...models import MessageLog
from ..utils import foreignkey
from ..base import WeChatModelAdmin


class MessageLogAdmin(WeChatModelAdmin):
    __category__ = "messagelog"
    __model__ = MessageLog

    list_display = (
        "msg_id", foreignkey("user"), "type", "content", "created_at")
    list_filter = ("type", )
    search_fields = (
        "=user__openid", "=user__unionid", "user__nickname", "user__comment",
        "content")

    fields = (
        "msg_id", foreignkey("user"), "type", "content", "created_at")
    readonly_fields = fields

    def has_add_permission(self, request):
        return False

    def get_model_perms(self, request):
        if not request.app.abilities.interactable:
            return {}
        return super(MessageLogAdmin, self).get_model_perms(request)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from contextlib import contextmanager

from django.http import response
from six.moves.urllib.parse import parse_qsl, urlparse
from six import text_type


@contextmanager
def mutable_GET(request):
    request.GET._mutable = True
    try:
        yield request.GET
    finally:
        request.GET._mutable = False


def get_ip(request):
    """获取客户端ip"""
    if not request:
        return None
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def get_params(request, key, default=None):
    """获取url上的参数"""
    if request.is_ajax():
        try:
            referrer = request.META["HTTP_REFERER"]
            query = dict(parse_qsl(urlparse(referrer).query))
            return query.get(key, default)
        except:
            return default
    else:
        return request.GET.get(key, default)


def auto_response(resp):
    if isinstance(resp, text_type):
        return response.HttpResponse(resp)
    elif isinstance(resp, dict):
        return response.JsonResponse(resp)
    else:
        return resp



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from unittest import mock
except ImportError:
    import mock

from wechat_django.models import WeChatApp
from wechat_django.tests.base import WeChatTestCaseBase
from ..models import WeChatPay


class WeChatPayTestCase(WeChatTestCaseBase):
    @classmethod
    def setUpTestData(cls):
        super(WeChatPayTestCase, cls).setUpTestData()
        app = WeChatApp.objects.create(
            title="pay", name="pay", appid="appid", appsecret="secret")
        pay = WeChatPay.objects.create(
            app=app, mch_id="mch_id", api_key="api_key", mch_cert=b"mch_cert",
            mch_key=b"mch_key")

        app_nocert = WeChatApp.objects.create(
            title="pay_nocert", name="pay_nocert", appid="pay_nocert",
            appsecret="secret")
        pay = WeChatPay.objects.create(
            app=app_nocert, mch_id="mch_id", api_key="api_key")

        miniprogram = WeChatApp.objects.create(
            title="miniprogram", name="miniprogram", appid="miniprogram",
            appsecret="secret", type=WeChatApp.Type.MINIPROGRAM)
        pay = WeChatPay.objects.create(
            app=miniprogram, mch_id="mch_id", api_key="api_key",
            mch_cert=b"mch_cert", mch_key=b"mch_key")

        app_sub = WeChatApp.objects.create(
            title="pay_sub", name="pay_sub", appid="pay_sub_appid",
            appsecret="secret")
        pay = WeChatPay.objects.create(
            app=app_sub, mch_id="mch_id", api_key="api_key",
            sub_mch_id="sub_mch_id", mch_app_id=app.appid,
            mch_cert=b"mch_cert", mch_key=b"mch_key")

    def setUp(self):
        self.app = WeChatApp.objects.get_by_name("pay")
        self.miniprogram = WeChatApp.objects.get_by_name("miniprogram")
        self.app_sub = WeChatApp.objects.get_by_name("pay_sub")
        self.app_nocert = WeChatApp.objects.get_by_name("pay_nocert")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings

SITE_HOST = getattr(settings, "WECHAT_SITE_HOST", None)
SITE_HTTPS = getattr(settings, "WECHAT_SITE_HTTPS", True)

PATCHADMINSITE = getattr(settings, "WECHAT_PATCHADMINSITE", True)

SESSIONSTORAGE = getattr(
    settings, "WECHAT_SESSIONSTORAGE", "django.core.cache.cache")

WECHATCLIENT = getattr(
    settings, "WECHAT_WECHATCLIENT", "wechat_django.client.WeChatClient")

OAUTHCLIENT = getattr(
    settings, "WECHAT_OAUTHCLIENT", "wechat_django.oauth.WeChatOAuthClient")

MESSAGETIMEOFFSET = getattr(settings, "WECHAT_MESSAGETIMEOFFSET", 180)

MESSAGENOREPEATNONCE = getattr(settings, "WECHAT_MESSAGENOREPEATNONCE", True)
